# coding=utf-8

import requests
import sys
import os
import russell
from russell.manager.auth_config import AuthConfigManager
from russell.exceptions import *
from russell.log import logger as russell_logger
import json
import tarfile
import zipfile

class RussellHttpClient(object):
    """
    Base client for all HTTP operations
    """
    def __init__(self):
        self.base_url = "{}/api/v1".format(russell.russell_host)
        self.access_token = AuthConfigManager.get_access_token()

    def request(self,
                method,
                url,
                params=None,
                data=None,
                files=None,
                timeout=5,
                access_token=None,
                stream=False):
        """
        Execute the request using requests library
        """
        request_url = self.base_url + url
        russell_logger.debug("Starting request to url: {} with params: {}, data: {}".format(request_url, params, data))
        if access_token:
            headers = {"Authorization": "Basic {}".format(access_token)}
        else:
            headers = {"Authorization": "Basic {}".format(
                self.access_token.token if self.access_token else None)
            }

        try:
            # print "url: {}".format(request_url)
            # print "params: {}".format(params)
            # print "data: {}".format(data)
            response = requests.request(method,
                                        request_url,
                                        params=params,
                                        headers=headers,
                                        data=data,
                                        files=files,
                                        timeout=timeout,
                                        stream=stream)
        except requests.exceptions.ConnectionError:
            sys.exit("Cannot connect to the Russell server. Check your internet connection.")

        if not stream:
            try:
                russell_logger.debug("Response Content: {}, Headers: {}".format(response.json(), response.headers))
            except Exception:
                russell_logger.debug("Request failed. Response: {}".format(response.content))
            self.check_response_status(response)
            return response.json().get("data","")
        else:
            russell_logger.info('HTTP Stream Request/Response...')
            self.check_response_status(response)
            return response

    def check_response_status(self, response):
        """
        Check if response is successful. Else raise Exception.
        """
        if not response.headers.get('Content-Type') == 'application/json':
            return
        if not (200 <= response.status_code < 300 and response.json().get("code") != None):
            if response.status_code == 401:
                raise AuthenticationException()
            elif response.status_code == 401:
                raise NotFoundException()
            else:
                raise InvalidResponseException()

        code = response.json()["code"]
        if not (200 <= code < 300):
            try:
                message = response.json().get("data","")
            except Exception:
                message = None
            russell_logger.debug("Error received : status_code: {}, message: {}".format(code,
                                                                                      message or response.content))
            if code == 517:
                raise NotFoundException()
            elif code == 512:
                raise AuthenticationException()
            elif code == 519:
                raise BadRequestException()
            elif code == 518:
                raise NoRequestException()
            elif code == 520:
                raise OverLimitException()
            elif code == 522:
                raise OverPermissionException()
            else:
                response.raise_for_status()

    def download(self, url, filename, timeout=10):
        """
        Download the file from the given url at the current path
        """
        russell_logger.debug("Downloading file from url: {}".format(url))

        try:
            # response = requests.get(request_url,
            #                         headers=request_headers,
            #                         timeout=timeout,
            #                         stream=True)
            response = self.request(method='GET',
                                    url=url,
                                    stream=True,
                                    timeout=timeout
                                    )
            self.check_response_status(response)
            with open(filename, 'wb') as f:
                for chunk in response.iter_content(chunk_size=1024):
                    if chunk:
                        f.write(chunk)
            return filename
        except requests.exceptions.ConnectionError as exception:
            russell_logger.debug("Exception: {}".format(exception))
            sys.exit("Cannot connect to the Russell server. Check your internet connection.")

    def download_compressed(self, url, compression='tar', uncompress=True, delete_after_uncompress=False, dir=None):
        """
        Download and optionally uncompress the tar file from the given url
        """
        if dir:
            if os.path.exists(dir):
                raise ExistedException
            else:
                os.mkdir(dir)
                os.chdir(dir)
        try:
            russell_logger.info("Downloading the tar file to the current directory ...")
            filename = self.download(url=url, filename='output')
            if filename and os.path.isfile(filename) and uncompress:
                russell_logger.info("uncompressring the contents of the file ...")
                if compression == 'tar':
                    tar = tarfile.open(filename)
                    tar.extractall()
                    tar.close()
                elif compression == 'zip':
                    zip = zipfile.ZipFile(filename)
                    zip.extractall()
                    zip.close()
            if delete_after_uncompress:
                russell_logger.info("Cleaning up the compressed file ...")
                os.remove(filename)
            return filename
        except requests.exceptions.ConnectionError as e:
            russell_logger.info("Download ERROR! {}".format(e))
            return False